==> navigine_sdk-1.2.1%2B3.tar.gz: lib/models/zone.dart <==
part of navigine_sdk;

class Zone {
  int id = 0;
  int location = 0;
  int subLocation = 0;
  String name = "";
  String color = "";
  String alias = "";
  List<LocationPoint> points = <LocationPoint>[];
  static final double epsilon = 1.0E-8;
  static final double delta = 1.0E-4;

  static Zone fromMap(data) {
    Zone z = Zone();
    z.id = data["id"];
    z.location = data["location"];
    z.subLocation = data["subLocation"];
    z.name = data["name"];
    z.color = data["color"];
    z.alias = data["alias"];

    for (var point in data["points"]) {
      z.points.add(LocationPoint.fromMap(point));
    }

    return z;
  }

  bool isValid() {
    return this.id > 0 && this.points.length > 2;
  }

  bool contains(LocationPoint p) {
    if (this.location == p.location &&
        this.subLocation == p.subLocation &&
        this.points.length >= 3) {
      int n = this.points.length;
      int count = 0;

      for (int i = 0; i < n; ++i) {
        LocationPoint p1 = this.points[i];
        LocationPoint p2 = this.points[(i + 1) % n];
        if (pointOnLine(p.x, p.y, p1.x, p1.y, p2.x, p2.y)) {
          return true;
        }

        if (xRayIntersectsLine(p.x, p.y, p1.x, p1.y, p2.x, p2.y)) {
          ++count;
        }
      }

      return count % 2 == 1;
    } else {
      return false;
    }
  }

  LocationPoint center() {
    if (!this.isValid()) {
      return LocationPoint();
    } else {
      double xsum = 0.0;
      double ysum = 0.0;
      double perim = 0.0;

      for (int i = 0; i < this.points.length; ++i) {
        LocationPoint p = this.points[i];
        LocationPoint q = this.points[(i + 1) % this.points.length];
        double length = p.distanceTo(q);
        perim += length;
        xsum += (p.x + q.x) * length / 2.0;
        ysum += (p.y + q.y) * length / 2.0;
      }

      return perim > 1.0E-8
          ? LocationPoint.fromData(
              this.location, this.subLocation, xsum / perim, ysum / perim)
          : LocationPoint.fromLocationPoint(this.points[0]);
    }
  }

  static bool pointOnLine(
      double x, double y, double x1, double y1, double x2, double y2) {
    double ax = x1 - x;
    double ay = y1 - y;
    double bx = x2 - x;
    double by = y2 - y;
    return (ax * by - ay * bx).abs() < 1.0E-8 && ax * bx + ay * by < 1.0E-8;
  }

  static bool xRayIntersectsLine(
      double x, double y, double x1, double y1, double x2, double y2) {
    if ((y - y1).abs() < 1.0E-8) {
      y1 += 1.0E-4;
    }

    if ((y - y2).abs() < 1.0E-8) {
      y2 += 1.0E-4;
    }

    double det;
    if (y1 < y && y < y2) {
      det = x2 * (y - y1) + x1 * (y2 - y) + x * (y1 - y2);
      return det + 1.0E-8 > 0.0;
    } else if (y2 < y && y < y1) {
      det = x2 * (y1 - y) + x1 * (y - y2) + x * (y2 - y1);
      return det + 1.0E-8 > 0.0;
    } else {
      return false;
    }
  }
}

==> navigine_sdk-1.2.4%2B3.tar.gz: lib/models/zone.dart <==
part of navigine_sdk;

class Zone {
  int id = 0;
  int location = 0;
  int subLocation = 0;
  String name = "";
  String color = "";
  String alias = "";
  List<LocationPoint> points = <LocationPoint>[];
  static final num epsilon = 1.0E-8;
  static final num delta = 1.0E-4;

  static Zone fromMap(data) {
    Zone z = Zone();
    z.id = data["id"];
    z.location = data["location"];
    z.subLocation = data["subLocation"];
    z.name = data["name"];
    z.color = data["color"];
    z.alias = data["alias"];

    for (var point in data["points"]) {
      z.points.add(LocationPoint.fromMap(point));
    }

    return z;
  }

  bool isValid() {
    return this.id > 0 && this.points.length > 2;
  }

  bool contains(LocationPoint p) {
    if (this.location == p.location &&
        this.subLocation == p.subLocation &&
        this.points.length >= 3) {
      int n = this.points.length;
      int count = 0;

      for (int i = 0; i < n; ++i) {
        LocationPoint p1 = this.points[i];
        LocationPoint p2 = this.points[(i + 1) % n];
        if (pointOnLine(p.x, p.y, p1.x, p1.y, p2.x, p2.y)) {
          return true;
        }

        if (xRayIntersectsLine(p.x, p.y, p1.x, p1.y, p2.x, p2.y)) {
          ++count;
        }
      }

      return count % 2 == 1;
    } else {
      return false;
    }
  }

  LocationPoint center() {
    if (!this.isValid()) {
      return LocationPoint();
    } else {
      num xsum = 0.0;
      num ysum = 0.0;
      num perim = 0.0;

      for (int i = 0; i < this.points.length; ++i) {
        LocationPoint p = this.points[i];
        LocationPoint q = this.points[(i + 1) % this.points.length];
        num length = p.distanceTo(q);
        perim += length;
        xsum += (p.x + q.x) * length / 2.0;
        ysum += (p.y + q.y) * length / 2.0;
      }

      return perim > 1.0E-8
          ? LocationPoint.fromData(
              this.location, this.subLocation, xsum / perim, ysum / perim)
          : LocationPoint.fromLocationPoint(this.points[0]);
    }
  }

  static bool pointOnLine(
    num x,
    num y,
    num x1,
    num y1,
    num x2,
    num y2,
  ) {
    num ax = x1 - x;
    num ay = y1 - y;
    num bx = x2 - x;
    num by = y2 - y;
    return (ax * by - ay * bx).abs() < 1.0E-8 && ax * bx + ay * by < 1.0E-8;
  }

  static bool xRayIntersectsLine(
    num x,
    num y,
    num x1,
    num y1,
    num x2,
    num y2,
  ) {
    if ((y - y1).abs() < 1.0E-8) {
      y1 += 1.0E-4;
    }

    if ((y - y2).abs() < 1.0E-8) {
      y2 += 1.0E-4;
    }

    num det;
    if (y1 < y && y < y2) {
      det = x2 * (y - y1) + x1 * (y2 - y) + x * (y1 - y2);
      return det + 1.0E-8 > 0.0;
    } else if (y2 < y && y < y1) {
      det = x2 * (y1 - y) + x1 * (y - y2) + x * (y2 - y1);
      return det + 1.0E-8 > 0.0;
    } else {
      return false;
    }
  }
}
